#!/bin/sh -e

if [ -z "$ARCH" ]; then
  ARCH=`dpkg --print-architecture`
fi

if [ -z "$ARCHT" ]; then
  case $ARCH in
  i386) ARCHT=i386-linux-gnu ;;
  amd64) ARCHT=x86_64-linux-gnu ;;
  *) echo "unknown arch" ; exit 1 ;;
  esac
fi

if [ ! -L ./build/libfwm.so ]; then
  echo "compile first!"
  exit
fi

SONAME=`readlink ./build/libfwm.so`

rm -Rf ./build/lib.deb ./build/dev.deb

mkdir ./build/lib.deb ./build/dev.deb

# place files
mkdir -p ./build/lib.deb/usr/lib/$ARCHT ./build/dev.deb/usr/lib/$ARCHT ./build/dev.deb/usr/include/$ARCHT/fwm
cp -p ./build/$SONAME ./build/lib.deb/usr/lib/$ARCHT/$SONAME
ln -s $SONAME ./build/dev.deb/usr/lib/$ARCHT/libfwm.so
cp -p libfwm.h ./build/dev.deb/usr/include/$ARCHT/fwm/

# place controls
VERSION=`head -1 ./CHANGELOG.TXT | tr -d -`
SIZELIB=`du -k -s ./build/lib.deb | awk '{print $1}'`
SIZEDEV=`du -k -s ./build/dev.deb | awk '{print $1}'`
VER1=`dpkg-query -W -f='${Version}' libc6`
VER2=`dpkg-query -W -f='${Version}' libx11-6`
mkdir ./build/lib.deb/DEBIAN ./build/dev.deb/DEBIAN
( cd ./build/lib.deb && md5deep -r -l usr) > ./build/lib.deb/DEBIAN/md5sums
( cd ./build/dev.deb && md5deep -r -l usr) > ./build/dev.deb/DEBIAN/md5sums 2>> /dev/null
cat > ./build/lib.deb/DEBIAN/control <<EOF
Package: libfwm
Version: $VERSION
Architecture: $ARCH
Maintainer: firk <firk@cantconnect.ru>
Installed-Size: $SIZELIB
Depends: libc6 (>= $VER1), libx11-6 (>= $VER2)
Section: libs
Priority: optional
Multi-Arch: same
Homepage: https://dev.m1089.ru/fwmx
Description: firk's window manager library
 fwm is a window manager for the X Window System.
EOF
cat > ./build/dev.deb/DEBIAN/control <<EOF
Package: libfwm-dev
Version: $VERSION
Architecture: $ARCH
Maintainer: firk <firk@cantconnect.ru>
Installed-Size: $SIZEDEV
Depends: libfwm (>= $VERSION)
Section: libdevel
Priority: optional
Multi-Arch: same
Homepage: https://dev.m1089.ru/fwmx
Description: firk's window manager library - development files
 fwm is a window manager for the X Window System.
EOF

fakeroot dpkg-deb --build ./build/lib.deb ./build
fakeroot dpkg-deb --build ./build/dev.deb ./build

rm -Rf ./build/lib.deb ./build/dev.deb

echo "DONE"
