#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

static Display *display;
static Screen *screen;
static Window win;
static unsigned long colors[16];
static GC gc;
static unsigned int x0, y0, W0, H0, bw;

static void init_colors(void) {
  XColor c;
  unsigned int n;
  for(n=0;n<16;n++) {
    c.flags = DoRed|DoGreen|DoBlue;
    c.red = ((n&8)?0x5500:0) + ((n&4)?0xAA00:0);
    c.green = ((n&8)?0x5500:0) + ((n&2)?0xAA00:0);
    c.blue = ((n&8)?0x5500:0) + ((n&1)?0xAA00:0);
    if(n==6) c.green=0x5500;
    if(!XAllocColor(display, DefaultColormapOfScreen(screen), &c)) { fprintf(stderr, "color[%d] alloc failed\n", n); exit(-1); }
    colors[n] = c.pixel;
  }
}

extern unsigned long base_color(unsigned char c) {
  return colors[c&15];
}

static void dump_info(void) {
  int n, j;
  n = ScreenCount(display);
  printf("ScreenCount = %d\n", n);
  for(j=0; j<n; j++) {
    screen = ScreenOfDisplay(display, j);
    printf("  screen[%d] %dx%d\n", j+1, WidthOfScreen(screen), HeightOfScreen(screen));
  }
}

int main(int argc, char **argv) {
  XEvent ev;
  XSetWindowAttributes wa;

  if(!(display = XOpenDisplay(NULL))) { fprintf(stderr, "XOpenDisplay(NULL) failed\n"); exit(-1); }
  dump_info();

  if(!(screen = DefaultScreenOfDisplay(display))) { fprintf(stderr, "DefaultScreenOfDisplay() failed\n"); exit(-1); }
  init_colors();

  x0 = 100; y0 = 100; W0 = 100; H0 = 100; bw = 10;

  wa.override_redirect = True;
  wa.event_mask = ExposureMask|ButtonPressMask|ButtonReleaseMask|PointerMotionMask|LeaveWindowMask;
  wa.background_pixel = base_color(0);
  wa.border_pixel = base_color(15);
  win = XCreateWindow(display, DefaultRootWindow(display),
    x0, y0, W0, H0, bw,
    CopyFromParent, CopyFromParent, CopyFromParent,
    CWOverrideRedirect|CWEventMask|CWBackPixel|CWBorderPixel, &wa);

  XSetStandardProperties(display, win, "TEST", "TEST", None, NULL, 0, NULL);

  XSelectInput(display, win,
       ExposureMask|                                 // window redraw requests
       ButtonPressMask|ButtonReleaseMask|            // mouse buttons click
       KeyPressMask|KeyReleaseMask|KeymapStateMask|  // keyboard clicks / resync pressed state on getting focus
       PointerMotionMask|                            // mouse moving
       StructureNotifyMask                           // window moving/resizing
       );
//  XSelectInput(display, win, -1);
  gc = XCreateGC(display, win, 0, NULL);
  XSetForeground(display, gc, base_color(15));
  XSetBackground(display, gc, base_color(0));
  XClearWindow(display, win);
  XMapRaised(display, win);
  XSync(display, False);

  while(1) {
    XNextEvent(display, &ev);
    if(ev.type==MotionNotify || ev.type==Expose) continue;
    if(ev.type==ButtonPress) dump_info();
    printf("ev.type = %d\n", ev.type);
    switch(ev.type) {
    case ConfigureNotify:
      printf("ConfigureNotify win=%ld base=%dx%d size=%dx%d\n",
          (long)ev.xconfigure.window, ev.xconfigure.x, ev.xconfigure.y, ev.xconfigure.width, ev.xconfigure.height);
      break;
    }
  }

  XCloseDisplay(display);
  return 0;
}
